@@ -46,6 +46,11 @@ struct private_file_logger_t {
 	 * strftime() format of time prefix, if any
 	 */
 	char *time_format;
+
+	/**
+	 * Print the name/# of the IKE_SA?
+	 */
+	bool ike_name;
 };
 
 /**
@@ -56,7 +61,7 @@ static bool log_(private_file_logger_t *this, debug_t group, level_t level,
 {
 	if (level <= this->levels[group])
 	{
-		char buffer[8192], timestr[128];
+		char buffer[8192], timestr[128], namestr[128] = "";
 		char *current = buffer, *next;
 		struct tm tm;
 		time_t t;
@@ -67,6 +72,23 @@ static bool log_(private_file_logger_t *this, debug_t group, level_t level,
 			localtime_r(&t, &tm);
 			strftime(timestr, sizeof(timestr), this->time_format, &tm);
 		}
+		if (this->ike_name && ike_sa)
+		{
+			if (ike_sa->get_peer_cfg(ike_sa))
+			{
+				snprintf(namestr, sizeof(namestr), " <%s|%d>",
+					ike_sa->get_name(ike_sa), ike_sa->get_unique_id(ike_sa));
+			}
+			else
+			{
+				snprintf(namestr, sizeof(namestr), " <%d>",
+					ike_sa->get_unique_id(ike_sa));
+			}
+		}
+		else
+		{
+			namestr[0] = '\0';
+		}
 
 		/* write in memory buffer first */
 		vsnprintf(buffer, sizeof(buffer), format, args);
@@ -81,13 +103,13 @@ static bool log_(private_file_logger_t *this, debug_t group, level_t level,
 			}
 			if (this->time_format)
 			{
-				fprintf(this->out, "%s %.2d[%N] %s\n",
-						timestr, thread, debug_names, group, current);
+				fprintf(this->out, "%s %.2d[%N]%s %s\n",
+						timestr, thread, debug_names, group, namestr, current);
 			}
 			else
 			{
-				fprintf(this->out, "%.2d[%N] %s\n",
-						thread, debug_names, group, current);
+				fprintf(this->out, "%.2d[%N]%s %s\n",
+						thread, debug_names, group, namestr, current);
 			}
 			current = next;
 		}
@@ -129,7 +151,7 @@ static void destroy(private_file_logger_t *this)
 /*
  * Described in header.
  */
-file_logger_t *file_logger_create(FILE *out, char *time_format)
+file_logger_t *file_logger_create(FILE *out, char *time_format, bool ike_name)
 {
 	private_file_logger_t *this = malloc_thing(private_file_logger_t);
 
@@ -142,6 +164,7 @@ file_logger_t *file_logger_create(FILE *out, char *time_format)
 	/* private variables */
 	this->out = out;
 	this->time_format = time_format;
+	this->ike_name = ike_name;
 	set_level(this, DBG_ANY, LEVEL_SILENT);
 
 	return &this->public;
