@@ -36,7 +36,7 @@ struct private_load_tester_ipsec_t {
 
 METHOD(kernel_ipsec_t, get_spi, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
-	   protocol_id_t protocol, u_int32_t reqid, u_int32_t *spi)
+	   u_int8_t protocol, u_int32_t reqid, u_int32_t *spi)
 {
 	*spi = ++this->spi;
 	return SUCCESS;
@@ -51,7 +51,7 @@ METHOD(kernel_ipsec_t, get_cpi, status_t,
 
 METHOD(kernel_ipsec_t, add_sa, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
-	   u_int32_t spi, protocol_id_t protocol, u_int32_t reqid, mark_t mark,
+	   u_int32_t spi, u_int8_t protocol, u_int32_t reqid, mark_t mark,
 	   lifetime_cfg_t *lifetime, u_int16_t enc_alg, chunk_t enc_key,
 	   u_int16_t int_alg, chunk_t int_key, ipsec_mode_t mode, u_int16_t ipcomp,
 	   u_int16_t cpi, bool encap, bool inbound, traffic_selector_t *src_ts,
@@ -61,7 +61,7 @@ METHOD(kernel_ipsec_t, add_sa, status_t,
 }
 
 METHOD(kernel_ipsec_t, update_sa, status_t,
-	   private_load_tester_ipsec_t *this, u_int32_t spi, protocol_id_t protocol,
+	   private_load_tester_ipsec_t *this, u_int32_t spi, u_int8_t protocol,
 	   u_int16_t cpi, host_t *src, host_t *dst, host_t *new_src,
 	   host_t *new_dst, bool encap, bool new_encap, mark_t mark)
 {
@@ -70,14 +70,14 @@ METHOD(kernel_ipsec_t, update_sa, status_t,
 
 METHOD(kernel_ipsec_t, query_sa, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
-	   u_int32_t spi, protocol_id_t protocol, mark_t mark, u_int64_t *bytes)
+	   u_int32_t spi, u_int8_t protocol, mark_t mark, u_int64_t *bytes)
 {
 	return NOT_SUPPORTED;
 }
 
 METHOD(kernel_ipsec_t, del_sa, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
-	   u_int32_t spi, protocol_id_t protocol, u_int16_t cpi, mark_t mark)
+	   u_int32_t spi, u_int8_t protocol, u_int16_t cpi, mark_t mark)
 {
 	return SUCCESS;
 }
@@ -85,9 +85,8 @@ METHOD(kernel_ipsec_t, del_sa, status_t,
 METHOD(kernel_ipsec_t, add_policy, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
 	   traffic_selector_t *src_ts, traffic_selector_t *dst_ts,
-	   policy_dir_t direction, u_int32_t spi, protocol_id_t protocol,
-	   u_int32_t reqid, mark_t mark, ipsec_mode_t mode, u_int16_t ipcomp,
-	   u_int16_t cpi, bool routed)
+	   policy_dir_t direction, policy_type_t type, ipsec_sa_cfg_t *sa,
+	   mark_t mark, bool routed)
 {
 	return SUCCESS;
 }
