@@ -354,7 +354,7 @@ static void terminate_srcip(private_stroke_control_t *this,
 		}
 
 		/* schedule delete asynchronously */
-		charon->processor->queue_job(charon->processor, (job_t*)
+		lib->processor->queue_job(lib->processor, (job_t*)
 						delete_ike_sa_job_create(ike_sa->get_id(ike_sa), TRUE));
 	}
 	enumerator->destroy(enumerator);
