@@ -45,8 +45,10 @@ ENUM(builder_part_names, BUILD_FROM_FILE, BUILD_END,
 	"BUILD_PATHLEN",
 	"BUILD_X509_FLAG",
 	"BUILD_REVOKED_ENUMERATOR",
-	"BUILD_SMARTCARD_KEYID",
-	"BUILD_SMARTCARD_PIN",
+	"BUILD_CHALLENGE_PWD",
+	"BUILD_PKCS11_MODULE",
+	"BUILD_PKCS11_SLOT",
+	"BUILD_PKCS11_KEYID",
 	"BUILD_RSA_MODULUS",
 	"BUILD_RSA_PUB_EXP",
 	"BUILD_RSA_PRIV_EXP",
