@@ -227,7 +227,7 @@ static field_desc isat_fields_ah[] = {
 	{ ft_mbz, 8/BITS_PER_BYTE, NULL, NULL },
 	{ ft_len, 16/BITS_PER_BYTE, "length", NULL },
 	{ ft_nat, 8/BITS_PER_BYTE, "transform number", NULL },
-	{ ft_enum, 8/BITS_PER_BYTE, "transform ID", &ah_transformid_names },
+	{ ft_enum, 8/BITS_PER_BYTE, "transform ID", &ah_transform_names },
 	{ ft_mbz, 16/BITS_PER_BYTE, NULL, NULL },
 	{ ft_end, 0, NULL, NULL }
 };
@@ -242,7 +242,7 @@ static field_desc isat_fields_esp[] = {
 	{ ft_mbz, 8/BITS_PER_BYTE, NULL, NULL },
 	{ ft_len, 16/BITS_PER_BYTE, "length", NULL },
 	{ ft_nat, 8/BITS_PER_BYTE, "transform number", NULL },
-	{ ft_enum, 8/BITS_PER_BYTE, "transform ID", &esp_transformid_names },
+	{ ft_enum, 8/BITS_PER_BYTE, "transform ID", &esp_transform_names },
 	{ ft_mbz, 16/BITS_PER_BYTE, NULL, NULL },
 	{ ft_end, 0, NULL, NULL }
 };
@@ -535,7 +535,7 @@ struct_desc isakmp_vendor_id_desc = { "ISAKMP Vendor ID Payload", isag_fields, s
 	 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 	 ! Next Payload  !   RESERVED    !         Payload Length        !
 	 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-	 !     Type      !   RESERVED    !           Identifier          ! 
+	 !     Type      !   RESERVED    !           Identifier          !
 	 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 	 !                                                               !
 	 ~                           Attributes                          ~
