@@ -471,7 +471,7 @@ enum {
 	DBGOPT_EMITTING,    /* same order as DBG_* */
 	DBGOPT_CONTROL,     /* same order as DBG_* */
 	DBGOPT_LIFECYCLE,   /* same order as DBG_* */
-	DBGOPT_KLIPS,       /* same order as DBG_* */
+	DBGOPT_KERNEL,      /* same order as DBG_* */
 	DBGOPT_DNS,         /* same order as DBG_* */
 	DBGOPT_NATT,        /* same order as DBG_* */
 	DBGOPT_OPPO,        /* same order as DBG_* */
@@ -659,7 +659,8 @@ static const struct option long_opts[] = {
 	{ "debug-emitting", no_argument, NULL, DBGOPT_EMITTING + OO },
 	{ "debug-control", no_argument, NULL, DBGOPT_CONTROL + OO },
 	{ "debug-lifecycle", no_argument, NULL, DBGOPT_LIFECYCLE + OO },
-	{ "debug-klips", no_argument, NULL, DBGOPT_KLIPS + OO },
+	{ "debug-klips", no_argument, NULL, DBGOPT_KERNEL + OO },
+	{ "debug-kernel", no_argument, NULL, DBGOPT_KERNEL + OO },
 	{ "debug-dns", no_argument, NULL, DBGOPT_DNS + OO },
 	{ "debug-natt", no_argument, NULL, DBGOPT_NATT + OO },
 	{ "debug-oppo", no_argument, NULL, DBGOPT_OPPO + OO },
@@ -1595,7 +1596,7 @@ int main(int argc, char **argv)
 		case DBGOPT_EMITTING:   /* --debug-emitting */
 		case DBGOPT_CONTROL:    /* --debug-control */
 		case DBGOPT_LIFECYCLE:  /* --debug-lifecycle */
-		case DBGOPT_KLIPS:      /* --debug-klips */
+		case DBGOPT_KERNEL:     /* --debug-kernel, --debug-klips */
 		case DBGOPT_DNS:        /* --debug-dns */
 		case DBGOPT_NATT:       /* --debug-natt */
 		case DBGOPT_OPPO:       /* --debug-oppo */
